@@ -26,33 +26,34 @@ var (
 const DefaultMarkerName = ".stfolder"
 
 type FolderConfiguration struct {
-	ID                    string                      `xml:"id,attr" json:"id"`
-	Label                 string                      `xml:"label,attr" json:"label" restart:"false"`
-	FilesystemType        fs.FilesystemType           `xml:"filesystemType" json:"filesystemType"`
-	Path                  string                      `xml:"path,attr" json:"path"`
-	Type                  FolderType                  `xml:"type,attr" json:"type"`
-	Devices               []FolderDeviceConfiguration `xml:"device" json:"devices"`
-	RescanIntervalS       int                         `xml:"rescanIntervalS,attr" json:"rescanIntervalS"`
-	FSWatcherEnabled      bool                        `xml:"fsWatcherEnabled,attr" json:"fsWatcherEnabled"`
-	FSWatcherDelayS       int                         `xml:"fsWatcherDelayS,attr" json:"fsWatcherDelayS"`
-	IgnorePerms           bool                        `xml:"ignorePerms,attr" json:"ignorePerms"`
-	AutoNormalize         bool                        `xml:"autoNormalize,attr" json:"autoNormalize"`
-	MinDiskFree           Size                        `xml:"minDiskFree" json:"minDiskFree"`
-	Versioning            VersioningConfiguration     `xml:"versioning" json:"versioning"`
-	Copiers               int                         `xml:"copiers" json:"copiers"` // This defines how many files are handled concurrently.
-	PullerMaxPendingKiB   int                         `xml:"pullerMaxPendingKiB" json:"pullerMaxPendingKiB"`
-	Hashers               int                         `xml:"hashers" json:"hashers"` // Less than one sets the value to the number of cores. These are CPU bound due to hashing.
-	Order                 PullOrder                   `xml:"order" json:"order"`
-	IgnoreDelete          bool                        `xml:"ignoreDelete" json:"ignoreDelete"`
-	ScanProgressIntervalS int                         `xml:"scanProgressIntervalS" json:"scanProgressIntervalS"` // Set to a negative value to disable. Value of 0 will get replaced with value of 2 (default value)
-	PullerPauseS          int                         `xml:"pullerPauseS" json:"pullerPauseS"`
-	MaxConflicts          int                         `xml:"maxConflicts" json:"maxConflicts"`
-	DisableSparseFiles    bool                        `xml:"disableSparseFiles" json:"disableSparseFiles"`
-	DisableTempIndexes    bool                        `xml:"disableTempIndexes" json:"disableTempIndexes"`
-	Paused                bool                        `xml:"paused" json:"paused"`
-	WeakHashThresholdPct  int                         `xml:"weakHashThresholdPct" json:"weakHashThresholdPct"` // Use weak hash if more than X percent of the file has changed. Set to -1 to always use weak hash.
-	MarkerName            string                      `xml:"markerName" json:"markerName"`
-	UseLargeBlocks        bool                        `xml:"useLargeBlocks" json:"useLargeBlocks"`
+	ID                      string                      `xml:"id,attr" json:"id"`
+	Label                   string                      `xml:"label,attr" json:"label" restart:"false"`
+	FilesystemType          fs.FilesystemType           `xml:"filesystemType" json:"filesystemType"`
+	Path                    string                      `xml:"path,attr" json:"path"`
+	Type                    FolderType                  `xml:"type,attr" json:"type"`
+	Devices                 []FolderDeviceConfiguration `xml:"device" json:"devices"`
+	RescanIntervalS         int                         `xml:"rescanIntervalS,attr" json:"rescanIntervalS" default:"3600"`
+	FSWatcherEnabled        bool                        `xml:"fsWatcherEnabled,attr" json:"fsWatcherEnabled" default:"true"`
+	FSWatcherDelayS         int                         `xml:"fsWatcherDelayS,attr" json:"fsWatcherDelayS" default:"10"`
+	IgnorePerms             bool                        `xml:"ignorePerms,attr" json:"ignorePerms"`
+	AutoNormalize           bool                        `xml:"autoNormalize,attr" json:"autoNormalize" default:"true"`
+	MinDiskFree             Size                        `xml:"minDiskFree" json:"minDiskFree" default:"1%"`
+	Versioning              VersioningConfiguration     `xml:"versioning" json:"versioning"`
+	Copiers                 int                         `xml:"copiers" json:"copiers"` // This defines how many files are handled concurrently.
+	PullerMaxPendingKiB     int                         `xml:"pullerMaxPendingKiB" json:"pullerMaxPendingKiB"`
+	Hashers                 int                         `xml:"hashers" json:"hashers"` // Less than one sets the value to the number of cores. These are CPU bound due to hashing.
+	Order                   PullOrder                   `xml:"order" json:"order"`
+	IgnoreDelete            bool                        `xml:"ignoreDelete" json:"ignoreDelete"`
+	ScanProgressIntervalS   int                         `xml:"scanProgressIntervalS" json:"scanProgressIntervalS"` // Set to a negative value to disable. Value of 0 will get replaced with value of 2 (default value)
+	PullerPauseS            int                         `xml:"pullerPauseS" json:"pullerPauseS"`
+	MaxConflicts            int                         `xml:"maxConflicts" json:"maxConflicts" default:"-1"`
+	DisableSparseFiles      bool                        `xml:"disableSparseFiles" json:"disableSparseFiles"`
+	DisableTempIndexes      bool                        `xml:"disableTempIndexes" json:"disableTempIndexes"`
+	Paused                  bool                        `xml:"paused" json:"paused"`
+	WeakHashThresholdPct    int                         `xml:"weakHashThresholdPct" json:"weakHashThresholdPct"` // Use weak hash if more than X percent of the file has changed. Set to -1 to always use weak hash.
+	MarkerName              string                      `xml:"markerName" json:"markerName"`
+	UseLargeBlocks          bool                        `xml:"useLargeBlocks" json:"useLargeBlocks" default:"true"`
+	CopyOwnershipFromParent bool                        `xml:"copyOwnershipFromParent" json:"copyOwnershipFromParent"`
 
 	cachedFilesystem fs.Filesystem
 
@@ -68,18 +69,15 @@ type FolderDeviceConfiguration struct {
 
 func NewFolderConfiguration(myID protocol.DeviceID, id, label string, fsType fs.FilesystemType, path string) FolderConfiguration {
 	f := FolderConfiguration{
-		ID:               id,
-		Label:            label,
-		RescanIntervalS:  3600,
-		FSWatcherEnabled: true,
-		FSWatcherDelayS:  10,
-		MinDiskFree:      Size{Value: 1, Unit: "%"},
-		Devices:          []FolderDeviceConfiguration{{DeviceID: myID}},
-		AutoNormalize:    true,
-		MaxConflicts:     -1,
-		FilesystemType:   fsType,
-		Path:             path,
+		ID:             id,
+		Label:          label,
+		Devices:        []FolderDeviceConfiguration{{DeviceID: myID}},
+		FilesystemType: fsType,
+		Path:           path,
 	}
+
+	util.SetDefaults(&f)
+
 	f.prepare()
 	return f
 }
@@ -108,7 +106,7 @@ func (f FolderConfiguration) Versioner() versioner.Versioner {
 	}
 	versionerFactory, ok := versioner.Factories[f.Versioning.Type]
 	if !ok {
-		l.Fatalf("Requested versioning type %q that does not exist", f.Versioning.Type)
+		panic(fmt.Sprintf("Requested versioning type %q that does not exist", f.Versioning.Type))
 	}
 
 	return versionerFactory(f.ID, f.Filesystem(), f.Versioning.Params)
@@ -280,7 +278,7 @@ func (f *FolderConfiguration) CheckAvailableSpace(req int64) error {
 	}
 	usage.Free -= req
 	if usage.Free > 0 {
-		if err := checkFreeSpace(f.MinDiskFree, usage); err == nil {
+		if err := CheckFreeSpace(f.MinDiskFree, usage); err == nil {
 			return nil
 		}
 	}
