@@ -12,23 +12,16 @@ import (
 	"github.com/syndtr/goleveldb/leveldb/util"
 )
 
-// Flush batches to disk when they contain this many records.
-const batchFlushSize = 64
-
 // A readOnlyTransaction represents a database snapshot.
 type readOnlyTransaction struct {
-	*leveldb.Snapshot
-	db *instance
+	snapshot
+	keyer keyer
 }
 
 func (db *instance) newReadOnlyTransaction() readOnlyTransaction {
-	snap, err := db.GetSnapshot()
-	if err != nil {
-		panic(err)
-	}
 	return readOnlyTransaction{
-		Snapshot: snap,
-		db:       db,
+		snapshot: db.GetSnapshot(),
+		keyer:    db.keyer,
 	}
 }
 
@@ -37,7 +30,52 @@ func (t readOnlyTransaction) close() {
 }
 
 func (t readOnlyTransaction) getFile(folder, device, file []byte) (protocol.FileInfo, bool) {
-	return t.db.getFile(t.db.keyer.GenerateDeviceFileKey(nil, folder, device, file))
+	return t.getFileByKey(t.keyer.GenerateDeviceFileKey(nil, folder, device, file))
+}
+
+func (t readOnlyTransaction) getFileByKey(key []byte) (protocol.FileInfo, bool) {
+	if f, ok := t.getFileTrunc(key, false); ok {
+		return f.(protocol.FileInfo), true
+	}
+	return protocol.FileInfo{}, false
+}
+
+func (t readOnlyTransaction) getFileTrunc(key []byte, trunc bool) (FileIntf, bool) {
+	bs, err := t.Get(key, nil)
+	if err == leveldb.ErrNotFound {
+		return nil, false
+	}
+	if err != nil {
+		l.Debugln("surprise error:", err)
+		return nil, false
+	}
+	f, err := unmarshalTrunc(bs, trunc)
+	if err != nil {
+		l.Debugln("unmarshal error:", err)
+		return nil, false
+	}
+	return f, true
+}
+
+func (t readOnlyTransaction) getGlobal(keyBuf, folder, file []byte, truncate bool) ([]byte, FileIntf, bool) {
+	keyBuf = t.keyer.GenerateGlobalVersionKey(keyBuf, folder, file)
+
+	bs, err := t.Get(keyBuf, nil)
+	if err != nil {
+		return keyBuf, nil, false
+	}
+
+	vl, ok := unmarshalVersionList(bs)
+	if !ok {
+		return keyBuf, nil, false
+	}
+
+	keyBuf = t.keyer.GenerateDeviceFileKey(keyBuf, folder, vl.Versions[0].Device, file)
+	if fi, ok := t.getFileTrunc(keyBuf, truncate); ok {
+		return keyBuf, fi, true
+	}
+
+	return keyBuf, nil, false
 }
 
 // A readWriteTransaction is a readOnlyTransaction plus a batch for writes.
@@ -45,14 +83,13 @@ func (t readOnlyTransaction) getFile(folder, device, file []byte) (protocol.File
 // batch size.
 type readWriteTransaction struct {
 	readOnlyTransaction
-	*leveldb.Batch
+	*batch
 }
 
 func (db *instance) newReadWriteTransaction() readWriteTransaction {
-	t := db.newReadOnlyTransaction()
 	return readWriteTransaction{
-		readOnlyTransaction: t,
-		Batch:               new(leveldb.Batch),
+		readOnlyTransaction: db.newReadOnlyTransaction(),
+		batch:               db.newBatch(),
 	}
 }
 
@@ -61,39 +98,20 @@ func (t readWriteTransaction) close() {
 	t.readOnlyTransaction.close()
 }
 
-func (t readWriteTransaction) checkFlush() {
-	if t.Batch.Len() > batchFlushSize {
-		t.flush()
-		t.Batch.Reset()
-	}
-}
-
-func (t readWriteTransaction) flush() {
-	if err := t.db.Write(t.Batch, nil); err != nil {
-		panic(err)
-	}
-}
-
-func (t readWriteTransaction) insertFile(fk, folder, device []byte, file protocol.FileInfo) {
-	l.Debugf("insert; folder=%q device=%v %v", folder, protocol.DeviceIDFromBytes(device), file)
-
-	t.Put(fk, mustMarshal(&file))
-}
-
 // updateGlobal adds this device+version to the version list for the given
 // file. If the device is already present in the list, the version is updated.
 // If the file does not have an entry in the global list, it is created.
-func (t readWriteTransaction) updateGlobal(gk, folder, device []byte, file protocol.FileInfo, meta *metadataTracker) bool {
+func (t readWriteTransaction) updateGlobal(gk, keyBuf, folder, device []byte, file protocol.FileInfo, meta *metadataTracker) ([]byte, bool) {
 	l.Debugf("update global; folder=%q device=%v file=%q version=%v invalid=%v", folder, protocol.DeviceIDFromBytes(device), file.Name, file.Version, file.IsInvalid())
 
 	var fl VersionList
 	if svl, err := t.Get(gk, nil); err == nil {
 		fl.Unmarshal(svl) // Ignore error, continue with empty fl
 	}
-	fl, removedFV, removedAt, insertedAt := fl.update(folder, device, file, t.db)
+	fl, removedFV, removedAt, insertedAt := fl.update(folder, device, file, t.readOnlyTransaction)
 	if insertedAt == -1 {
 		l.Debugln("update global; same version, global unchanged")
-		return false
+		return keyBuf, false
 	}
 
 	name := []byte(file.Name)
@@ -102,19 +120,25 @@ func (t readWriteTransaction) updateGlobal(gk, folder, device []byte, file proto
 	if insertedAt == 0 {
 		// Inserted a new newest version
 		global = file
-	} else if new, ok := t.getFile(folder, fl.Versions[0].Device, name); ok {
-		global = new
 	} else {
-		panic("This file must exist in the db")
+		keyBuf = t.keyer.GenerateDeviceFileKey(keyBuf, folder, fl.Versions[0].Device, name)
+		if new, ok := t.getFileByKey(keyBuf); ok {
+			global = new
+		} else {
+			// This file must exist in the db, so this must be caused
+			// by the db being closed - bail out.
+			l.Debugln("File should exist:", name)
+			return keyBuf, false
+		}
 	}
 
 	// Fixup the list of files we need.
-	t.updateLocalNeed(folder, name, fl, global)
+	keyBuf = t.updateLocalNeed(keyBuf, folder, name, fl, global)
 
 	if removedAt != 0 && insertedAt != 0 {
 		l.Debugf(`new global for "%v" after update: %v`, file.Name, fl)
 		t.Put(gk, mustMarshal(&fl))
-		return true
+		return keyBuf, true
 	}
 
 	// Remove the old global from the global size counter
@@ -125,7 +149,8 @@ func (t readWriteTransaction) updateGlobal(gk, folder, device []byte, file proto
 		// The previous newest version is now at index 1
 		oldGlobalFV = fl.Versions[1]
 	}
-	if oldFile, ok := t.getFile(folder, oldGlobalFV.Device, name); ok {
+	keyBuf = t.keyer.GenerateDeviceFileKey(keyBuf, folder, oldGlobalFV.Device, name)
+	if oldFile, ok := t.getFileByKey(keyBuf); ok {
 		// A failure to get the file here is surprising and our
 		// global size data will be incorrect until a restart...
 		meta.removeFile(protocol.GlobalDeviceID, oldFile)
@@ -137,24 +162,25 @@ func (t readWriteTransaction) updateGlobal(gk, folder, device []byte, file proto
 	l.Debugf(`new global for "%v" after update: %v`, file.Name, fl)
 	t.Put(gk, mustMarshal(&fl))
 
-	return true
+	return keyBuf, true
 }
 
-// updateLocalNeeds checks whether the given file is still needed on the local
+// updateLocalNeed checks whether the given file is still needed on the local
 // device according to the version list and global FileInfo given and updates
 // the db accordingly.
-func (t readWriteTransaction) updateLocalNeed(folder, name []byte, fl VersionList, global protocol.FileInfo) {
-	nk := t.db.keyer.GenerateNeedFileKey(nil, folder, name)
-	hasNeeded, _ := t.db.Has(nk, nil)
+func (t readWriteTransaction) updateLocalNeed(keyBuf, folder, name []byte, fl VersionList, global protocol.FileInfo) []byte {
+	keyBuf = t.keyer.GenerateNeedFileKey(keyBuf, folder, name)
+	hasNeeded, _ := t.Has(keyBuf, nil)
 	if localFV, haveLocalFV := fl.Get(protocol.LocalDeviceID[:]); need(global, haveLocalFV, localFV.Version) {
 		if !hasNeeded {
 			l.Debugf("local need insert; folder=%q, name=%q", folder, name)
-			t.Put(nk, nil)
+			t.Put(keyBuf, nil)
 		}
 	} else if hasNeeded {
 		l.Debugf("local need delete; folder=%q, name=%q", folder, name)
-		t.Delete(nk)
+		t.Delete(keyBuf)
 	}
+	return keyBuf
 }
 
 func need(global FileIntf, haveLocal bool, localVersion protocol.Vector) bool {
@@ -176,54 +202,62 @@ func need(global FileIntf, haveLocal bool, localVersion protocol.Vector) bool {
 // removeFromGlobal removes the device from the global version list for the
 // given file. If the version list is empty after this, the file entry is
 // removed entirely.
-func (t readWriteTransaction) removeFromGlobal(gk, folder, device, file []byte, meta *metadataTracker) {
+func (t readWriteTransaction) removeFromGlobal(gk, keyBuf, folder, device []byte, file []byte, meta *metadataTracker) []byte {
 	l.Debugf("remove from global; folder=%q device=%v file=%q", folder, protocol.DeviceIDFromBytes(device), file)
 
 	svl, err := t.Get(gk, nil)
 	if err != nil {
 		// We might be called to "remove" a global version that doesn't exist
 		// if the first update for the file is already marked invalid.
-		return
+		return keyBuf
 	}
 
 	var fl VersionList
 	err = fl.Unmarshal(svl)
 	if err != nil {
 		l.Debugln("unmarshal error:", err)
-		return
+		return keyBuf
 	}
 
 	fl, _, removedAt := fl.pop(device)
 	if removedAt == -1 {
 		// There is no version for the given device
-		return
+		return keyBuf
 	}
 
 	if removedAt == 0 {
 		// A failure to get the file here is surprising and our
 		// global size data will be incorrect until a restart...
-		if f, ok := t.getFile(folder, device, file); ok {
+		keyBuf = t.keyer.GenerateDeviceFileKey(keyBuf, folder, device, file)
+		if f, ok := t.getFileByKey(keyBuf); ok {
 			meta.removeFile(protocol.GlobalDeviceID, f)
 		}
 	}
 
 	if len(fl.Versions) == 0 {
-		t.Delete(t.db.keyer.GenerateNeedFileKey(nil, folder, file))
+		keyBuf = t.keyer.GenerateNeedFileKey(keyBuf, folder, file)
+		t.Delete(keyBuf)
 		t.Delete(gk)
-		return
+		return keyBuf
 	}
 
 	if removedAt == 0 {
-		global, ok := t.getFile(folder, fl.Versions[0].Device, file)
+		keyBuf = t.keyer.GenerateDeviceFileKey(keyBuf, folder, fl.Versions[0].Device, file)
+		global, ok := t.getFileByKey(keyBuf)
 		if !ok {
-			panic("This file must exist in the db")
+			// This file must exist in the db, so this must be caused
+			// by the db being closed - bail out.
+			l.Debugln("File should exist:", file)
+			return keyBuf
 		}
-		t.updateLocalNeed(folder, file, fl, global)
+		keyBuf = t.updateLocalNeed(keyBuf, folder, file, fl, global)
 		meta.addFile(protocol.GlobalDeviceID, global)
 	}
 
 	l.Debugf("new global after remove: %v", fl)
 	t.Put(gk, mustMarshal(&fl))
+
+	return keyBuf
 }
 
 func (t readWriteTransaction) deleteKeyPrefix(prefix []byte) {
